@@ -2,6 +2,8 @@ import ipywidgets as widgets
 import random as rd
 import numpy as np
 import sympy
+import copy
+from IPython.display import Image
 
 
 ##########
@@ -1581,9 +1583,9 @@ Flux v7 = {}\n\n
         print("Concounter=",concounter)
         print("Glucose/v1",self.v1)
         print("Lactate/v4",self.v4)
-        print("Formate/v5",self.v5)
-        print("Acetate/v6",self.v6)
-        print("Ethanol/v7",self.v7)
+        print("v5",self.v5)
+        print("v6",self.v6)
+        print("v7",self.v7)
         print("Nonconform pairs:",noncon)
         print("2v1-v4-v5 = 0 | {} +- {}".format((p1[0]-p2[0]),(p1[1]+p2[1])))
         print("2v1-v4-2v6 = 0 | {} +- {}".format((p1[0]-p3[0]),(p1[1]+p3[1])))
@@ -1672,4 +1674,519 @@ This task is to determine if the observed flux values are consistent or not.\nFo
 Example:\nv_Acetate - v_Ethanol = 0 |  (5.0 - 5.5) +/- (0.25 + 0.275) = 0\n 
 ##########\n
             """#HHAVE TO HAVE ANOTHE RLOOK AT THIS
-            print(txt)
\ No newline at end of file
+            print(txt)
+            
+##########
+##########
+##########
+#class for homework 7
+##########
+##########
+##########
+class create_question_homework_seven():
+    #Innit: a question is comprised of the actual
+    #Question (string)
+    #and a list of answer_options (string) and an int if they are true (1) or not (0)
+    def __init__(self, question = str, answers = [], image = None):
+        if image != None:
+            try:
+                file = open(image, "rb")
+                image = file.read()
+                self.image_widget = self.create_image_widget(image)
+            except:
+                print("No image to load!")
+                self.image_widget = None
+        else:
+            self.image_widget = None
+        self.question = question
+        self.answerobjects = []
+        self.create_answer_option_objects(answers)
+        #self.answerobjects = self.create_answer_option_objects()
+        #self.answerobjects2 = []
+        
+    #create a label with the question
+    def create_question_label(self):
+        self.question_label  = widgets.Label(value=self.question)
+        return
+    
+    def create__verify_button(self):
+        self.verify_button = widgets.Button(
+            description='Verify answer',
+            disabled=False,
+            button_style='warning', # 'success', 'info', 'warning', 'danger' or ''
+            tooltip='Click here to verify your answer!',
+            icon='check' # (FontAwesome names without the `fa-` prefix)
+        )
+        return
+    
+    def create_image_widget(self, image):
+        i =  widgets.Image(
+            value=image,
+            format='png',
+            width=300,
+            height=300,
+        )
+        return i
+    
+    #def create_answer_option_objects(self):
+    #    list_of_objects = []
+    #    for i in self.answers:
+    #        y = create_answer_option(i[0],i[1])
+    #        list_of_objects.append(y)
+    #    return list_of_objects
+    
+    def create_answer_option_objects(self, original_list_of_answers):
+        list_of_answers = copy.deepcopy(original_list_of_answers)
+        x = len(list_of_answers)
+        if x == 1:
+            self.answerobjects.append(create_answer_option(list_of_answers[0]))
+            return
+        if x == 0:
+            print('Something went wrong')
+            return
+        else:
+            y = rd.randint(0,x-1)
+            self.answerobjects.append(create_answer_option(list_of_answers[y]))
+            del list_of_answers[y]
+            self.create_answer_option_objects(list_of_answers)
+            return
+    
+    def check_answer_option_objects(self):
+        for i in self.answerobjects:
+            i.check_anwer_option()
+        return
+    
+    def get_full_question(self):
+        l = []
+        self.create_question_label()
+        self.create__verify_button()
+        l.append(self.question_label)
+        
+        if self.image_widget != None:
+            l.append(self.image_widget)
+        
+        for i in self.answerobjects:
+            l.append(i.button)
+        
+        l.append(self.verify_button)
+        wbox = widgets.VBox(l)
+        return wbox
+    
+    def on_button_clicked(self, click):
+        self.verify_button.disabled = True
+        self.check_answer_option_objects()
+
+class create_answer_option():
+    #this takes the list of answer options of a question and turns it into toogle.button widgets
+    #Position 0 in the list is the descripion
+    #Position 1 in the list is if the answer option is True(1) or false (0)
+    #Position 2 in the list is for the case that it's not a True-False question.
+        #this is why len(inputlist) is checked
+        
+    #Note: this is backwards and should be exchanged through a *args but the def calling these
+    # create_answer_option objects needs to call the positions in the greater list/dict, so that isnt possible (YET)
+    def __init__(self, inputlist):
+        self.descr = inputlist[0]
+        self.yesorno = inputlist[1]
+        
+        if len(inputlist) == 3:
+            optionlist = inputlist[2]
+        else:
+            optionlist = [("True",1),("False",0)]
+        
+        self.optionlist = optionlist
+        self.button = self.create_answer_option_button()
+
+    #create a toggle button for a single answer_option
+    def create_answer_option_button(self):
+        w = widgets.ToggleButtons(
+            options= self.optionlist,
+            description=self.descr,
+            disabled=False,
+            button_style='', # 'success', 'info', 'warning', 'danger' or ''
+           # tooltips=['Description of slow', 'Description of regular', 'Description of fast'],
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto', height='40px')
+        )
+        return w
+    
+    def check_anwer_option(self):
+        if self.button.value == self.yesorno:
+            if self.button.value == 1:
+                self.button.icons = ["check",""]
+            if self.button.value == 0:
+                self.button.icons = ["","check"]
+            self.button.button_style = 'success'
+            self.button.disabled = True
+        elif self.button.value != self.yesorno:
+            if self.button.value == 1:
+                self.button.icons = ["","check"]
+            if self.button.value == 0:
+                self.button.icons = ["check",""]
+            self.button.button_style = "danger"
+            self.button.disabled = True
+
+##########
+##########
+##########            
+class create_homework_seven_task():
+    def __init__(self, question, solution):
+        self.question = question
+        self.sol = solution
+        self.create_int_text()
+        self.create_question_label()
+        
+    def create_question_label(self):
+        self.question_label = widgets.Label(
+            value=self.question,
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto')    
+        )
+    
+    def create_int_text(self):
+        self.int_text = widgets.IntText(
+            value=0,
+            description='Answer:',
+            tooltip ='Answer:',
+            disabled=False,
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto', height='40px')
+        )
+    
+    def create_verify_button(self):
+        self.verify_button = widgets.Button(
+            description='Verify answer',
+            disabled=False,
+            button_style='warning', # 'success', 'info', 'warning', 'danger' or ''
+            tooltip='Click here to verify your answer!',
+            icon='check', # (FontAwesome names without the `fa-` prefix),
+            display='flex',
+            align_items='center',
+            flex_flow='column',
+            #align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='50%', height='40px')
+        )
+        
+    def on_verify_button_clicked(self, click):
+        if self.int_text.value == self.sol:
+            self.verify_button.disabled = True
+            self.int_text.disabled = True
+            self.verify_button.button_style = 'success'
+            self.verify_button.description="Correct answer!"
+        else:
+            #self.verify_button.disabled = True
+            #self.int_text.disabled = True
+            self.verify_button.button_style = 'danger'
+            self.verify_button.description="Wrong answer. Please try again!"
+            
+    def get_full_question(self):
+        self.create_verify_button()
+        #self.create_help_button()
+        #i = [self.verify_button, self.help_button]
+        #hbox = widgets.HBox(i)
+        
+        j = [self.question_label, self.int_text,self.verify_button]
+        wbox = widgets.VBox(j)
+        return wbox
+##########
+##########
+##########
+class create_homework_seven_task_one_one():
+    def __init__(self, matricnb = int):
+        self.init_display = self.build_init_display()
+        self.question =  "What is the maximum specific growth rate? [1/h]"
+        #self.create_solution() #needs self.ID from self.init_display #gets moved to on-click event in init-display
+        self.create_float_text()
+        self.create_question_label()
+        self.output = widgets.Output()
+        #self.ID created by on-click event on button in self.init_display
+        #self.sol created by  self.create_solution()
+    
+    def create_solution(self, ID):
+        if type(ID) == int:
+            rd.seed(ID)
+            listofrndms = [rd.random() for i in range(10)]
+            #self.sol = 0.300+listofrndms[9]*0.100
+            self.sol = 0.386
+        elif matricnb == "Test":
+            self.sol = 0.386
+        else:
+            with self.init_display.children[-1]:
+                print("matriculation number has to be an integer!")
+                #self.output.clear_output(wait=True)
+    
+    def create_question_label(self):
+        self.question_label = widgets.Label(
+            value=self.question,
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto')    
+        )
+    
+    def create_float_text(self):
+        # create two Float-TExt widgets that get implemented into the question interface
+        # self.float_corr is the correlation coefficient R
+        # self.float_mue is the growth rate µ
+        # both user-inputs can be checked by taking the value in 'widget'.value
+        self.float_corr = widgets.FloatText(
+            value=0,
+            description='Correlation Coefficient\n|| Example: R^2 = 0.00 (Float)',
+            tooltip ='Correlation Coefficient\n|| Example: R^2 = 0.00 (Float)',
+            disabled=False,
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto', height='40px')
+        )
+        self.float_mue = widgets.FloatText(
+            value=0,
+            description='maximum specific growthrate\n|| Example: µ = 1.00 (Float)',
+            tooltip ='maximum specific growthrate\n|| Example: µ = 1.00 (Float)',
+            disabled=False,
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto', height='40px')
+        )
+    
+    def create_verify_button(self):
+        self.verify_button = widgets.Button(
+            description='Verify answer',
+            disabled=False,
+            button_style='warning', # 'success', 'info', 'warning', 'danger' or ''
+            tooltip='Click here to verify your answer!',
+            icon='check', # (FontAwesome names without the `fa-` prefix),
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto', height='40px')
+        )
+            
+    def create_help_button(self):
+        self.help_button = widgets.Button(
+            description='Help required?',
+            disabled=False,
+            button_style='info', # 'success', 'info', 'warning', 'danger' or ''
+            tooltip='Click here to get a tipp!',
+            display='flex',
+            flex_flow='column',
+            align_items='stretch',
+            style= {'description_width': 'initial'},
+            layout = widgets.Layout(width='auto', height='40px')
+        )
+        
+    def on_verify_button_clicked(self, click):
+        self.verify_button.button_style = 'warning'
+        self.verify_button.description="Wrong answer. Please try again!"
+        #First step: Check if correlation coefficient is > 0.95
+        #If yes: move on to growth-rate
+        #if no: depict on verify button and return
+        if self.float_corr.value > 1.00:
+            self.verify_button.button_style = 'warning'
+            self.verify_button.description="Correlation Coefficient can't be > 1.00. Please try again!"
+        elif 0.95 <= self.float_corr.value <= 1.00:
+        #Second Step: check if growth rate solution is correct (is equal to self.sol) 
+        #If yes: answer correct
+        #If no: Answer wrong
+        #If close to correc: depict on verify button
+            if round(self.float_mue.value,2) == round(self.sol,2):
+                self.verify_button.disabled = True
+                self.float_corr.disabled = True
+                self.float_mue.disabled = True
+                self.verify_button.button_style = 'success'
+                self.verify_button.description="Correct answer!"
+                return
+            elif round(self.float_mue.value,1) == round(self.sol,1):
+                self.verify_button.button_style = 'danger'
+                self.verify_button.description="Wrong answer, but close! Please try again!"
+                return
+        else:
+            self.verify_button.button_style = 'warning'
+            self.verify_button.description="Correlation Coefficient has to be > 0.95. Please try again!"
+                
+            
+    def on_help_button_clicked(self, click):
+        self.help_button.disabled = True
+        txt = """
+##########\n
+For cells growing at a constant specific growth rate µ, the biomass concentration x increases with time:\n
+x_i = x_0 * e^m\n
+µ can be determined by plotting the natural logarithm of the OD against the time h\nand determining the slop of the exponential growth phase\n 
+µ = (ln(x_o) - ln(x_i)) / t\n\n
+Note:\n
+* To assume a linear correlation between µ and t, atleast three data points are required\n
+* The correlation coefficient R^2 has to be > 0.95 
+##########\n
+        """
+        with self.output:
+            print(txt)
+            
+    #on-click in init-display:
+    #store Student-ID in self.ID
+    def on_init_homework_button_clicked(self, click):
+        self.init_display.children[1].disabled = True
+        ID = self.init_display.children[0].value
+        self.create_solution(ID)
+        with self.init_display.children[2]:
+            print("#####\nStudent-ID accepted!\nGood luck!\n#####")
+
+    
+    def get_full_question(self):
+        self.create_verify_button()
+        self.create_help_button()
+        i = [self.verify_button, self.help_button]
+        hbox = widgets.HBox(i)
+        
+        j = [self.question_label, self.float_corr, self.float_mue, hbox, self.output]
+        wbox = widgets.VBox(j)
+        return wbox
+    
+    def build_init_display(self):
+        StudID = widgets.IntText( description='Student ID:', value=0 )
+        button_StudID   = widgets.Button( description='Generate Parameters' )
+        output = widgets.Output()
+
+        disp=widgets.VBox([StudID,button_StudID, output]) 
+        return disp
+
+###
+def full_question_objects():
+    q_1_1_answers = [
+        ["for butanol between 17 h - 32 h from the start of the process",0],
+        ["for neither butanol or butyrate",0],
+        ["for butyrate between 8 h - 14 h from the start of the process",1]    
+    ]
+    q_1_2_answers = [
+        ["is reached after complete glucose degradiation",0],
+        ["is 18.4 mM/h",1],
+        ["is reached before complete butyrate degradiation",1],
+        ["is 13.3 mM/h",0]
+    ]
+    q_1_3_1_answers = [
+        ["increases linearly",0],
+        ["decreeases linearly",0],
+        ["increases exponentially",1],
+        ["decreases exponentially",0],
+        ["stays constant",0]
+    ]
+    q_1_3_2_answers = [
+        ["stays the same",0],
+        ["exponentially",1],
+        ["linearly",0]
+    ]
+    q_1_3_3_answers = [
+        ["a linear increase in product concentration",0],
+        ["an exponential increase in product concentration",1]
+    ]
+    q_1_3_4_answers = [
+        ["the kinetic orders of both formation processes are the same",1],
+        ["the formation rates of both formation processes are the same",0],
+        ["product formation is growth-dependent",1],
+        ["product formation is growth-independent",0]
+    ]
+    q_2_1_answers = [
+        ["1+2 are organic solvents",0],
+        ["3+4 are organic solvents",1],
+        ["1+2 are organic acids",1],
+        ["3+4 are organic acids",0]
+    ]
+    q_2_2_answers = [
+        ["Acetyl-CoA ↔ Acetat + ATP\nAcetoacetyl-CoA + Butyrat → Aceton + CO2 + Butyryl-CoA",0],
+        ["Butyryl-CoA ↔ Butyrat + ATP\nAcetoacetyl-CoA + Acetat → Aceton + CO2 + Acetyl-CoA",0],
+        ["Butyryl-CoA ↔ Butyrat + ATP\nAcetoacetyl-CoA + Butyrat → Aceton + CO2 + Butyryl-CoA",1],
+        ["Acetate_extracellular ↔ Acetate_intracellular\nAcetoacetyl-CoA + Acetat → Aceton + CO2 + Acetyl-CoA",1],
+        ["Acetyl-CoA ↔ Acetat + ATP\nAcetoacetyl-CoA + Acetat → Aceton + CO2 + Acetyl-CoA",1]
+    ]
+    q_2_3_1_answers = [
+        ["not be determined by determining the concentration over time using HPLC,\nsince the metabolic network has too many degrees of freedom (is underdetermined).",1],
+        ["be determined by determining the concentration over time using HPLC.",0],
+        ["not be determined by determining concentration over time using HPLC since acetone is volatile.",0]
+    ]
+    q_2_3_2_answers = [
+        ["undetermined; not all intracellular rates can be calculated.",1],
+        ["over-determined; all intracellular rates can be calculated and error estimation performed.",0],
+        ["determined; all intracellular rates can be determined.",0]
+    ]
+    q_3_1_answers = [
+        ["Acetyl-CoA ↔ Acetat + ATP\nAcetoacetyl-CoA + Acetat → Aceton + CO2 + Acetyl-CoA",0],
+        ["Butyryl-CoA ↔ Butyrat + ATP",1],
+        ["Acetoacetyl-CoA → Acceton + CO2 + ATP",1],#added Aceton on Product side (Quant-MiBi_Calc-Guide.pdf, p. 30)
+        ["Acetyl-P ↔ Acetat + ATP",0],
+        ["Butyryl-CoA ↔ Butyrat + ATP\nAcetoacetyl-CoA + Butyrat → Aceton + CO2 + Butyryl-CoA",0],
+        ["Acetyl-CoA ↔ Acetat + ATP",1]
+    ]
+    q_3_2_answers = [
+        ["",0,[("Yes",1),("No",0)]]
+    ]
+    question_dict = {}
+    question_dict["q_1_1"] = ["The formation is directly proportional to the biomass increase:",q_1_1_answers, None]
+    question_dict["q_1_2"] = ["The maximum rate of butanol formation:",q_1_2_answers, None]
+    question_dict["q_1_3_1"] = ["If the biomass concentration increases exponentially over time, the biomass formation rate is:",q_1_3_1_answers, None]
+    question_dict["q_1_3_2"] = ["For cells that proliferate at a constant specific growth rate µ, the concentration of biomass increases over time:",q_1_3_2_answers, None]
+    question_dict["q_1_3_3"] = ["Product formation according to first-order kinetics corresponds to:",q_1_3_3_answers, None]
+    question_dict["q_1_3_4"] = ["If there is a linear relationship between the product and biomass concentration r_x:",q_1_3_4_answers, None]
+    question_dict["q_2_1"] = ["Which C. acetobutylicum products are represented by curves 1, 2, 3, and 4?",q_2_1_answers, "data/801059_Graph_Blank1997_Prozess_C_acetobutylicum_lora_080703_v3_modified4question.png"]
+    question_dict["q_2_2"] = ["Which reaction equations represent a possible synthetic route to acetone?",q_2_2_answers, None]
+    question_dict["q_2_3_1"] = ["The reaction rates to acetone can:",q_2_3_1_answers, None]
+    question_dict["q_2_3_2"] = ["The metabolic (stoichiometric) model is:",q_2_3_2_answers, None]
+    question_dict["q_3_1"] = ["Which reaction equations represent the net equations for butyrate, acetate and acetone?",q_3_1_answers, None]
+    question_dict["q_3_2"] = ["Is the acetone synthesis rate determinable?",q_3_2_answers, None]
+    
+    
+    list_of_objects = []
+    for i in question_dict.keys():
+        list_of_objects.append(create_question_homework_seven(question_dict[i][0],question_dict[i][1], question_dict[i][2]))
+    return list_of_objects
+
+def full_task_objects():
+    #t_1_1 = [
+    #    "What is the maximum specific growth rate? [1/h]",0.368
+    #]
+    t_2_1_1 = [
+        "How many reaction rates does the stoichiometric model representing the metabolic network of C. acetobutylicum contain?",20
+    ]
+    t_2_1_2 = [
+     "How many reaction pathways are there for the synthesis of acetone in C. acetobutylicum?",2   
+    ]
+    t_2_2_1 = [
+     "How many rates are in r_m?",8
+    ]
+    t_2_2_2 = [
+     "The number of rates r_c to calculate is:",12#is 12 correct?   
+    ]
+    t_2_2_3 = [
+     "How many of (all) the rates can be determined using liquid chromatography (HPLC with any detector) and a spectrophotometer?",8   
+    ]
+    t_3_1 = [
+     "How many reaction equations does the simplified metabolic model for calculating the intracellular reaction rates including acetone production contain?",11   
+    ]
+
+    task_dict = {}
+    #task_dict["t_1_1"] = t_1_1 #NOTE: Task 1.1 requires calculation and a float value as answer. because of this, it was kept seperate.
+    task_dict["t_2_1_1"] = t_2_1_1
+    task_dict["t_2_1_2"] = t_2_1_2
+    task_dict["t_2_2_1"] = t_2_2_1
+    task_dict["t_2_2_2"] = t_2_2_2
+    task_dict["t_2_2_3"] = t_2_2_3
+    task_dict["t_3_1"] = t_3_1
+    
+    
+    list_of_objects = []
+    for i in task_dict.keys():
+        list_of_objects.append(create_homework_seven_task(task_dict[i][0],task_dict[i][1]))
+    return list_of_objects
\ No newline at end of file
